//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//               Contains ENB Adaptation               //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// Boris: For ENBSeries and his knowledge and codes    //
// JawZ: Author and developer of this file             //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

  bool ENABLE_HLSENSE <
    string UIName = "Enable Hightlight Sensitivity";
  > = {false};

  int CHOOSE_OUTPUT <
    string UIName = "Choose Grey Output";  string UIWidget="Spinner";  int UIMin=1;  int UIMax=3;
  > = {1};
  float3 fRGB_Out <
    string UIName = "Alter Grey RGB Output nr2";  string UIWidget="Color";
  > = {0.3333, 0.3333, 0.3333};

float INFO5 <string UIName="--Instead of enbseries.ini values--";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
float EmptyRow1 <string UIName="1";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
  float4 fAdaptMin_E <
    string UIName = "ADAPTATION: Min [E]-";          string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=65536.0;
  > = {0.10, 0.10, 0.10, 0.10};

float EmptyRow2 <string UIName="2";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
  float4 fAdaptMax_E <
    string UIName = "ADAPTATION: Max [E]-";          string UIWidget="Spinner";  float UIMin=0.01;  float UIMax=65536.0;
  > = {10.0, 10.0, 10.0, 10.0};


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

///++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/// output size is 16*16
/// TextureCurrent size is 256*256, it's internally downscaled from full screen
/// input texture is R16G16B16A16 or R11G11B10 float format (alpha ignored)
/// output texture is R32 float format (red channel only)
///++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
float4 msDownsample(float4 inColor, float2 inCoord)
{
/// Time and Location interpolators ++++++++++++++++++++++++++++++++++++++++++++++++++++++///
//  float fAdaptSense = TODE1(TODfactors(), fAdaptSense_E.x, fAdaptSense_E.y, fAdaptSense_E.z, fAdaptSense_E.w);
///+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++///

/// Downsample 256*256 to 16*16
/// More complex blurring methods affect result if sensitivity uncommented
    float2 pos;
    float2 coord;
    float4 curr          = 0.0;
    float4 currmax       = 0.0;
    const float scale    = 1.0 / 16.0;
    const float step     = 1.0 / 16.0;
    const float halfstep = 0.5 / 16.0;
    pos.x                = -0.5 + halfstep;
    for (int x=0; x<16; x++)
    {
      pos.y = -0.5 + halfstep;
      for (int y=0; y<16; y++)
      {
        coord           = pos.xy * scale;
        float4 tempcurr = TextureCurrent.Sample(Sampler0, inCoord.xy + coord.xy);
        currmax         = max(currmax, tempcurr);
        curr           += tempcurr;
        pos.y          += step;
      }
      pos.x += step;
    }
    curr *= 1.0 / (16.0 * 16.0);

    inColor = curr;

  // Adjust sensitivity to small bright areas on the screen
  // Warning! Uncommenting the next line increases sensitivity a lot
  if (ENABLE_HLSENSE==true)    inColor = lerp(curr, currmax, AdaptationParameters.z); // fAdaptSense = AdaptationParameters.z

  ///TODO modify this math to your taste, for example lower intensity for blue colors
  ///gray output
  if (CHOOSE_OUTPUT==1)       inColor = AvgLuma(inColor.xyz).y;
  else if (CHOOSE_OUTPUT==2)  inColor = dot(inColor.xyz, float3(fRGB_Out.x, fRGB_Out.y, fRGB_Out.z));
  else if (CHOOSE_OUTPUT==3)  inColor = AvgLuma(inColor.xyz).w;

  return inColor;
}


///++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/// output size is 1*1
/// TexturePrevious size is 1*1
/// TextureCurrent size is 16*16
/// output and input textures are R32 float format (red channel only)
///++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
float4 msAdaptation(float4 inColor, float2 inCoord)
{
/// Time and Location interpolators ++++++++++++++++++++++++++++++++++++++++++++++++++++++///
//  float fAdaptSense = TODE1(TODfactors(), fAdaptSense_E.x, fAdaptSense_E.y, fAdaptSense_E.z, fAdaptSense_E.w);
//  float fAdaptTime  = TODE1(TODfactors(), fAdaptTime_E.x, fAdaptTime_E.y, fAdaptTime_E.z, fAdaptTime_E.w);
  float fAdaptMin   = TODE1(TODfactors(), fAdaptMin_E.x, fAdaptMin_E.y, fAdaptMin_E.z, fAdaptMin_E.w);
  float fAdaptMax   = TODE1(TODfactors(), fAdaptMax_E.x, fAdaptMax_E.y, fAdaptMax_E.z, fAdaptMax_E.w);
///+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++///

  float prev = TexturePrevious.Sample(Sampler0, inCoord.xy).x;

/// Downsample 16*16 to 1*1
    float2 pos;
    float curr           = 0.0;
    float currmax        = 0.0;
    const float step     = 1.0/16.0;
    const float halfstep = 0.5/16.0;
    pos.x                = halfstep;

    for (int x=0; x<16; x++)
    {
      pos.y = halfstep;
      for (int y=0; y<16; y++)
        {
          float tempcurr = TextureCurrent.Sample(Sampler0, inCoord.xy + pos.xy).x;
          currmax        = max(currmax, tempcurr);
          curr          += tempcurr;
          pos.y         += step;
        }
      pos.x += step;
    }
    curr *= 1.0/(16.0*16.0);

/// Adjust sensitivity to small bright areas on the screen
    curr = lerp(curr, currmax, AdaptationParameters.z); // fAdaptSense = AdaptationParameters.z

/// Smooth by time
    inColor = lerp(prev, curr, AdaptationParameters.w); // ~fAdaptTime * Timer.w

/// Clamp to avoid bugs in post process shader, which have much lower floating point precision
    inColor = max(inColor, 0.001);
    inColor = min(inColor, 16384.0);

/// Limit value if ForceMinMaxValues=true
    float valmax;
    float valcut;
    valmax   = max(inColor.x, max(inColor.y, inColor.z));
    valcut   = max(valmax, fAdaptMin); // fAdaptMin = AdaptationParameters.x
    valcut   = min(valcut, fAdaptMax); // fAdaptMax = AdaptationParameters.y
    inColor *= valcut/(valmax + 0.000000001f);

  return inColor;
}