//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                Contains Bloom Depth                 //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Prod80: Author of Adaptation Curve                  //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx or enbbloom.fx shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //




// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 enbBlurLO(Texture2D inputtex, float2 uvsrc, float srcsize, float destsize)
{
  const float scale=4.0;           /// Blurring range, samples count (performance) is factor of scale*scale
//  const float srcsize=1024.0;    /// In current example just blur input texture of 1024*1024 size
//  const float destsize=1024.0;   /// For last stage render target must be always 1024*1024

    float2 invtargetsize=scale/srcsize;
    invtargetsize.y*=ScreenSize.z; //correct by aspect ratio

    float2 fstepcount;
    fstepcount=srcsize;

    fstepcount*=invtargetsize;
    fstepcount=min(fstepcount, 16.0);
    fstepcount=max(fstepcount, 2.0);

    int stepcountX=(int)(fstepcount.x+0.4999);
    int stepcountY=(int)(fstepcount.y+0.4999);

    fstepcount=1.0/fstepcount;
    float4 curr=0.0;
    curr.w=0.000001;
    float2 pos;
    float2 halfstep=0.5*fstepcount.xy;
    pos.x=-0.5+halfstep.x;
    invtargetsize *= 2.0;
    for (int x=0; x<stepcountX; x++)
    {
      pos.y=-0.5+halfstep.y;
      for (int y=0; y<stepcountY; y++)
      {
        float2 coord=pos.xy * invtargetsize + uvsrc.xy;
        float3 tempcurr=inputtex.Sample(Sampler1, coord.xy).xyz;
        float  tempweight;
        float2 dpos=pos.xy*2.0;
        float  rangefactor=dot(dpos.xy, dpos.xy);
      // Loosing many pixels here, don't program such unefficient cycle yourself!
        tempweight=saturate(1001.0 - 1000.0*rangefactor);//arithmetic version to cut circle from square
        tempweight*=saturate(1.0 - rangefactor); //softness, without it bloom looks like bokeh dof
        curr.xyz+=tempcurr.xyz * tempweight;
        curr.w+=tempweight;

        pos.y+=fstepcount.y;
      }
      pos.x+=fstepcount.x;
    }
    curr.xyz/=curr.w;

//    curr.xyz=inputtex.Sample(Sampler1, uvsrc.xy);

  return curr.xyz;
}