//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//             Contains ENB Depth of Field             //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

  float EFocusingSensitivity <
    string UIName="Focus: sensitivity to nearest";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=1.0;
  > = {0.5};

  float EApertureSize <
    string UIName="Aperture: size";  string UIWidget="spinner";  float UIMin=1.0;  float UIMax=16.0;
  > = {2.0};

  float ESensorSize <  // kinda 1/CropFactor
    string UIName="Dof: sensor size";  string UIWidget="spinner";  float UIMin=4.8;  float UIMax=36.0; //35 mm full frame
  > = {36.0};

  float EBokehSoftness <
    string UIName="Dof: bokeh softness";  string UIWidget="spinner";  float UIMin=0.01;  float UIMax=1.0;
  > = {1.0};

  float EBlurRange <
    string UIName="Dof: blur max range"; string UIWidget="spinner"; float UIMin=0.1; float UIMax=2.0;
  > = {1.0};


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float4 msAperture(float4 inColor, float2 inCoord)
{
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//output size is 1*1
//TexturePrevious size is 1*1
//TextureCurrent not exist, so set to white 1.0
//output and input textures are R32 float format (red channel only)
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  float curr;
  float prev = TexturePrevious.Sample(Sampler0, inCoord.xy).x;

/// TODO compute aperture here, from adaptation or from global variables
    curr = EApertureSize;  /// Constant in this example
    curr = max(curr, 1.0); /// Safety
    curr = 1.0 / curr;     /// Map it to 0..1 range. for brightness it must be curr*curr (2*Pi*R*R)

// Smooth by time
    inColor = lerp(prev, curr, DofParameters.z); /// ApertureTime with elapsed time

// Clamp to avoid bugs, 1 means fully open, bigger than 0 for proper adaptation later
    inColor = max(inColor, 0.0000000001);
    inColor = min(inColor, 1.0);

  return inColor;
}

float4 msReadFocus(float4 inColor, float2 inCoord)
{
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//output size is 16*16
//output texture is R32 float format (red channel only)
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Warning! For first person models (weapon) it's better to ignore depth to avoid wrong focusing,
// so this is done via fpdistance

/// This example reads depth almost from center of the screen
    float2 pos;
    float  curr = 0.0;
//  float currmin = 1.0;
    const float step = 1.0 / 16.0;
    const float halfstep = 0.5 / 16.0;
    pos.x = halfstep;
    for (int x=0; x<16; x++)
    {
      pos.y = halfstep;
      for (int y=0; y<16; y++)
      {
        float2 coord = pos.xy * 0.05;
        coord += inCoord.xy * 0.05 + float2(0.5, 0.5); /// Somewhere around the center of screen
        float tempcurr = TextureDepth.SampleLevel(Sampler0, 0.5, 0.0).x;

      /// Do not blur first person models like weapons and hands
        const float fpdistance = 1.0 / 0.085;
        float fpfactor = 1.0-saturate(1.0 - tempcurr * fpdistance);
        tempcurr = lerp(1.0, tempcurr, fpfactor * fpfactor);

//      currmin = min(currmin, tempcurr);
        curr += tempcurr;

        pos.y += step;
      }
      pos.x += step;
    }
    curr *= 1.0 / (16.0 * 16.0);
    inColor = curr;

/// Clamp to avoid bugs
    inColor = max(inColor, 0.0);
    inColor = min(inColor, 1.0);

  return inColor;
}

float4 msFocus(float4 inColor, float2 inCoord)
{
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//output size is 1*1
//TexturePrevious size is 1*1
//TextureCurrent size is 16*16
//output and input textures are R32 float format (red channel only)
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    float prev = TexturePrevious.Sample(Sampler0, inCoord.xy).x;

/// Downsample 16*16 to 1*1
    float2 pos;
    float  curr          = 0.0;
    float  currmin       = 1.0;
    const float step     = 1.0 / 16.0;
    const float halfstep = 0.5 / 16.0;
    pos.x = halfstep;
    for (int x=0; x<16; x++)
    {
      pos.y = halfstep;
      for (int y=0; y<16; y++)
      {
        float tempcurr = TextureCurrent.Sample(Sampler0, inCoord.xy + pos.xy).x;
        currmin        = min(currmin, tempcurr);
        curr          +=tempcurr;
        pos.y         += step;
      }
      pos.x += step;
    }
    curr *= 1.0 / (16.0 * 16.0);

/// Adjust sensitivity to nearest areas of the screen
    curr = lerp(curr, currmin, EFocusingSensitivity);

/// Smooth by time
    inColor = lerp(prev, curr, DofParameters.w); /// FocusingTime with elapsed time

/// Clamp to avoid bugs, unless it's depth
    inColor = max(inColor, 0.0);
    inColor = min(inColor, 1.0);

  return inColor;
}

float4 msComputeFactor(float4 inColor, float2 inCoord)
{
// Draw to temporary render target

  float depth    = TextureDepth.Sample(Sampler0, inCoord.xy).x;
  float focus    = TextureFocus.Sample(Sampler0, inCoord.xy).x;
  float aperture = TextureAperture.Sample(Sampler0, inCoord.xy).x;

/// Clamp to avoid potenrial bugs
    depth = max(depth, 0.0);
    depth = min(depth, 1.0);

/// Compute blur radius
    float scaling = EBlurRange; /// Abstract scale in screen space
    float factor  = depth - focus;

    factor = factor * ESensorSize * aperture * scaling;
/// Limit size
    float screensizelimit = ESensorSize * scaling;
    factor = max(factor, -screensizelimit);
    factor = min(factor, screensizelimit);

    inColor = factor;

/// Do not blur first person models like weapons
    const float fpdistance = 1.0 / 0.085;
    float fpfactor         = 1.0 - saturate(1.0 - depth * fpdistance);
    inColor                = inColor * fpfactor * fpfactor;

  return inColor;
}

float4 msDoF(float4 inColor, float2 inCoord)
{
// Example of blur. without any fixes of artifacts and low performance

    float focusing;
    focusing = RenderTargetR16F.Sample(Sampler0, inCoord.xy).x;

    float2 sourcesizeinv;
    float2 fstepcount;
    sourcesizeinv   = ScreenSize.y;
    sourcesizeinv.y = ScreenSize.y*ScreenSize.z;
    fstepcount.x    = ScreenSize.x;
    fstepcount.y    = ScreenSize.x*ScreenSize.w;

    float2 pos;
    float2 coord;
    float4 curr   = 0.0;
    float  weight = 0.000001;

    fstepcount     = abs(focusing);
    sourcesizeinv *= focusing;

    fstepcount = min(fstepcount, 32.0);
    fstepcount = max(fstepcount, 0.0);

    int stepcountX = (int)(fstepcount.x + 1.4999);
    int stepcountY = (int)(fstepcount.y + 1.4999);
    fstepcount     = max(fstepcount, 2.0);
    float2 halfstepcountinv = 2.0 / fstepcount;
    pos.x = -1.0 + halfstepcountinv.x;
    for (int x=0; x<stepcountX; x++)
    {
      pos.y = -1.0 + halfstepcountinv.y;
      for (int y=0; y<stepcountY; y++)
      {
        float tempweight;
        float rangefactor = dot(pos.xy, pos.xy);
        coord             = pos.xy * sourcesizeinv;
        coord            += inCoord.xy;
        float4 tempcurr   = TextureColor.SampleLevel(Sampler1, coord.xy, 0.0);
        tempweight        = saturate(1001.0 - 1000.0*rangefactor);  /// Arithmetic version to cut circle from square
        tempweight       *= saturate(1.0 - rangefactor * EBokehSoftness);
        curr.xyz         += tempcurr.xyz * tempweight;
        weight           += tempweight;

        pos.y            += halfstepcountinv.y;
      }
      pos.x += halfstepcountinv.x;
    }
    curr.xyz /= weight;

    inColor.xyz = curr;

  return inColor;
}