//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//            Contains ENB Post-Process v1             //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Boris: Initial author of ENB post-process method    //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
  bool ENABLE_PP1 <
    string UIName = "Enable Post Process v1";
  > = {false};
//{
    float fAdaptMax <
        string UIName="TONEMAP: Adaptation Min";     string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=100.0;  float UIStep=0.001;
    > = {0.01};
    float fAdaptMin <
        string UIName="TONEMAP: Adaptation Max";     string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=100.0;  float UIStep=0.001;
    > = {0.07};
    float fContrast <
        string UIName="TONEMAP: Contrast";           string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {0.95};
    float fSaturation <
        string UIName="TONEMAP: Color Saturation";   string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {1.0};
    float fToneMappingOversaturation <
        string UIName="TONEMAP: Tonemapping Curve";  string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=50.0;
    > = {300.0};
    float fToneMappingCurve <
        string UIName="TONEMAP: Tonemapping Curve";  string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=50.0;
    > = {6.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 enbPP(float3 inColor, float3 inAdaptation)
{
  if (ENABLE_PP1==true)
  {
    float grayadaptation = AvgLuma(inAdaptation.xyz).y;
      grayadaptation     = max(grayadaptation, 0.0);
      grayadaptation     = min(grayadaptation, 50.0);
      inColor.xyz        = inColor.xyz / (grayadaptation * fAdaptMax + fAdaptMin);

    float cgray          = dot(inColor.xyz, float3(0.27, 0.67, 0.06));
    cgray                = pow(cgray, fContrast);
    float3 poweredcolor  = pow(inColor.xyz, fSaturation);
    float newgray        = dot(poweredcolor.xyz, float3(0.27, 0.67, 0.06));
      inColor.xyz        = poweredcolor.xyz * cgray / (newgray + 0.0001);

    float3 luma   = inColor.xyz;
    float lumamax = fToneMappingOversaturation;
      inColor.xyz = (inColor.xyz * (1.0 + inColor.xyz / lumamax)) / (inColor.xyz + fToneMappingCurve);
  }

  return inColor;
}