//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//            Contains ExtendedFX Sharpening           //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Boris: Initial Blurring and Sharpening code         //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffectpostpass.txt shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffectpostpass.txt file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_Row1 <
  string UIName="A--------------------------A";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};

  float EBlurAmount <
    string UIName="Blur: amount";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=1.0;
  > = {1.0};
  float EBlurRange <
    string UIName="Blur: range";    string UIWidget="spinner";  float UIMin=0.0;  float UIMax=2.0;
  > = {1.0};
  float ESharpAmount <
    string UIName="Sharp: amount";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=4.0;
  > = {1.0};
  float ESharpRange <
    string UIName="Sharp: range";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=2.0;
  > = {1.0};


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //



// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float4 enbBlur(float4 inColor, float2 inCoord)
{
  float4 color;
  float4 centercolor;
  float2 pixeloffset = ScreenSize.y;
  pixeloffset.y     *= ScreenSize.z;

    centercolor = TextureColor.Sample(Sampler0, inCoord.xy);
    color       = 0.0;
    float2 offsets[4]=
    {
      float2(-1.0,-1.0),
      float2(-1.0, 1.0),
      float2( 1.0,-1.0),
      float2( 1.0, 1.0),
    };
    for (int i=0; i<4; i++)
    {
      float2 coord = offsets[i].xy * pixeloffset.xy * EBlurRange + inCoord.xy;
      color.xyz += TextureColor.Sample(Sampler1, coord.xy);
    }
    color.xyz += centercolor.xyz;
    color.xyz *= 0.2;

    inColor.xyz = lerp(centercolor.xyz, color.xyz, EBlurAmount);

  return inColor;
}

float4 enbSharpen(float4 inColor, float2 inCoord)
{
  float4 color;
  float4 centercolor;
  float2 pixeloffset = ScreenSize.y;
  pixeloffset.y     *= ScreenSize.z;

    centercolor = TextureColor.Sample(Sampler0, inCoord.xy);
    color       = 0.0;
    float2 offsets[4]=
    {
      float2(-1.0,-1.0),
      float2(-1.0, 1.0),
      float2( 1.0,-1.0),
      float2( 1.0, 1.0),
    };
    for (int i=0; i<4; i++)
    {
      float2 coord = offsets[i].xy * pixeloffset.xy * ESharpRange + inCoord.xy;
      color.xyz   += TextureColor.Sample(Sampler1, coord.xy);
    }
    color.xyz *= 0.25;

    float diffgray = dot((centercolor.xyz - color.xyz), 0.3333);
    inColor.xyz    = ESharpAmount * centercolor.xyz * diffgray + centercolor.xyz;

  return inColor;
}
