//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                ENBSeries effect file                //
//         visit http://enbdev.com for updates         //
//       Copyright (c) 2007-2015 Boris Vorontsov       //
//----------------------ENB PRESET---------------------//
//               - Insert Preset Name -                //
//                - ENBSeries v0.289 -                 //
//-----------------------CREDITS-----------------------//
// Boris: For ENBSeries and his knowledge and codes    //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


float INFO1 <string UIName="-X - Sunrise";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
float INFO2 <string UIName="-Y - Day";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
float INFO3 <string UIName="-Z - Sunset";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
float INFO4 <string UIName="-W - Night";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};


//++++++++++++++++++++++++++++++++++++++++++++++++++++
// INTERNAL PARAMETERS BEGINS HERE, CAN BE MODIFIED
//++++++++++++++++++++++++++++++++++++++++++++++++++++




//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// EXTERNAL PARAMETERS BEGINS HERE, SHOULD NOT BE MODIFIED UNLESS YOU KNOW WHAT YOU ARE DOING
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

float4 Timer;            /// x = generic timer in range 0..1, period of 16777216 ms (4.6 hours), y = average fps, w = frame time elapsed (in seconds)
float4 ScreenSize;       /// x = Width, y = 1/Width, z = aspect, w = 1/aspect, aspect is Width/Height
float4 Weather;          /// x = current weather index, y = outgoing weather index, z = weather transition, w = time of the day in 24 standart hours. Weather index is value from weather ini file, for example WEATHER002 means index==2, but index==0 means that weather not captured.
float4 TimeOfDay1;       /// x = dawn, y = sunrise, z = day, w = sunset. Interpolators range from 0..1
float4 TimeOfDay2;       /// x = dusk, y = night. Interpolators range from 0..1
float  ENightDayFactor;  /// Changes in range 0..1, 0 means that night time, 1 - day time
float  EInteriorFactor;  /// Changes 0 or 1. 0 means that exterior, 1 - interior

// External ENB debugging paramaters
float4 tempF1;     /// 0,1,2,3  // Keyboard controlled temporary variables.
float4 tempF2;     /// 5,6,7,8  // Press and hold key 1,2,3...8 together with PageUp or PageDown to modify.
float4 tempF3;     /// 9,0
float4 tempInfo1;  /// xy = cursor position in range 0..1 of screen, z = is shader editor window active, w = mouse buttons with values 0..7
/// tempInfo1 assigned mouse button values
///    0 = none
///    1 = left
///    2 = right
///    3 = left+right
///    4 = middle
///    5 = left+middle
///    6 = right+middle
///    7 = left+right+middle (or rather cat is sitting on your mouse)
float4 tempInfo2;
// xy = cursor position of previous left mouse button click
// zw = cursor position of previous right mouse button click


// TEXTURES
Texture2D TextureDownsampled;   /// color R16B16G16A16 64 bit or R11G11B10 32 bit hdr format. 1024*1024 size
Texture2D TextureColor;         /// color which is output of previous technique (except when drawed to temporary render target), R16B16G16A16 64 bit hdr format. 1024*1024 size
Texture2D TextureOriginal;      /// color R16B16G16A16 64 bit or R11G11B10 32 bit hdr format, screen size. PLEASE AVOID USING IT BECAUSE OF ALIASING ARTIFACTS, UNLESS YOU FIX THEM
Texture2D TextureDepth;         /// scene depth R32F 32 bit hdr format, screen size. PLEASE AVOID USING IT BECAUSE OF ALIASING ARTIFACTS, UNLESS YOU FIX THEM
Texture2D TextureAperture;      /// this frame aperture 1*1 R32F hdr red channel only. computed in PS_Aperture of enbdepthoffield.fx

// Temporary textures which can be set as render target for techniques via annotations like <string RenderTarget="RenderTargetRGBA32";>
Texture2D RenderTarget1024;     /// R16B16G16A16F 64 bit hdr format, 1024*1024 size
Texture2D RenderTarget512;      /// R16B16G16A16F 64 bit hdr format, 512*512 size
Texture2D RenderTarget256;      /// R16B16G16A16F 64 bit hdr format, 256*256 size
Texture2D RenderTarget128;      /// R16B16G16A16F 64 bit hdr format, 128*128 size
Texture2D RenderTarget64;       /// R16B16G16A16F 64 bit hdr format, 64*64 size
Texture2D RenderTarget32;       /// R16B16G16A16F 64 bit hdr format, 32*32 size
Texture2D RenderTarget16;       /// R16B16G16A16F 64 bit hdr format, 16*16 size
Texture2D RenderTargetRGBA32;   /// R8G8B8A8 32 bit ldr format, screen size
Texture2D RenderTargetRGBA64F;  /// R16B16G16A16F 64 bit hdr format, screen size


// SAMPLERS
SamplerState Sampler0
{
  Filter=MIN_MAG_MIP_POINT;  AddressU=Clamp;  AddressV=Clamp;
};
SamplerState Sampler1
{
  Filter=MIN_MAG_MIP_LINEAR;  AddressU=Clamp;  AddressV=Clamp;
};


// DATA STRUCTURE
struct VS_INPUT_POST
{
  float3 pos     : POSITION;
  float2 txcoord : TEXCOORD0;
};
struct VS_OUTPUT_POST
{
  float4 pos      : SV_POSITION;
  float2 txcoord0 : TEXCOORD0;
};


// HELPER FUNCTIONS
/// Include the additional shader files containing all helper functions and constants
   #include "/Modular Shaders/msHelpers.fxh"  /// Required for all files below!
   #include "/Modular Shaders/enbbloom/01_BloomBlur.fxh"
   #include "/Modular Shaders/enbbloom/20_BloomDepth.fxh"
   #include "/Modular Shaders/enbbloom/21_BloomControl.fxh"

/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 *                                                                        *
 * - enbBlurLO(inputtex, IN.txcoord0.xy, srcsize, destsize)               *
 *                                                                        *
 * - msBloomDepth(res.xyz, IN.txcoord0.xy)                                *
 *                                                                        *
 * - msBloomControl(res.xyz, IN.txcoord0.xy)                              *
 **************************************************************************/


// VERTEX SHADER
VS_OUTPUT_POST VS_Quad(VS_INPUT_POST IN)
{
  VS_OUTPUT_POST OUT;

    float4 pos;
    pos.xyz=IN.pos.xyz;
    pos.w=1.0;
    OUT.pos=pos;
    OUT.txcoord0.xy=IN.txcoord.xy;

  return OUT;
}


// PIXEL SHADERS
/// Example 1. Draw in single pass directly to bloom texture of 1024*1024 size
/// Alpha channel is computed similar to allow perormance friendly "large blur" factor for Local Operators
float4 PS_ExampleBloom1(VS_OUTPUT_POST IN, float4 v0 : SV_Position0, uniform Texture2D inputtex, uniform float srcsize, uniform float destsize) : SV_Target
{
  float4 res;

    res.xyz = FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize);   /// Requires msHelpers.fxh active
    res.w   = enbBlurLO(inputtex, IN.txcoord0.xy, srcsize, destsize);  /// Requires 01_BloomBlur.fxh active, for Local Operators

    res.xyz = msBloomDepth(res.xyz, IN.txcoord0.xy);                   /// Requires 20_BloomDepth.fxh active

    res.xyz = msBloomControl(res.xyz, IN.txcoord0.xy);                 /// Requires 21_BloomControl.fxh active

/// Clamp to avoid bugs
    res = max(res, 0.0);
    res = min(res, 16384.0);

  return res;
}

/// Example 2. Draw in several passes to different render targets
/// Alpha channel is computed similar to allow perormance friendly "large blur" factor for Local Operators
float4 PS_Resize(VS_OUTPUT_POST IN, float4 v0 : SV_Position0, uniform Texture2D inputtex, uniform float srcsize, uniform float destsize) : SV_Target
{
  float4 res;

    res.xyz = FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize);   /// Requires msHelpers.fxh active
    res.w   = enbBlurLO(inputtex, IN.txcoord0.xy, srcsize, destsize);  /// Requires 01_BloomBlur.fxh active, for Local Operators

/// Clamp to avoid bugs
    res = max(res, 0.0);
    res = min(res, 16384.0);

  return res;
}

float4 PS_BloomMix(VS_OUTPUT_POST IN, float4 v0 : SV_Position0) : SV_Target
{
  float4 res;

/// Mix bloom textures
    res.xyz  = RenderTarget1024.Sample(Sampler1, IN.txcoord0.xy).xyz;
    res.xyz += RenderTarget512.Sample(Sampler1, IN.txcoord0.xy).xyz;
    res.xyz += RenderTarget256.Sample(Sampler1, IN.txcoord0.xy).xyz;
    res.xyz += RenderTarget128.Sample(Sampler1, IN.txcoord0.xy).xyz;
    res.xyz += RenderTarget64.Sample(Sampler1, IN.txcoord0.xy).xyz;
    res.xyz += RenderTarget32.Sample(Sampler1, IN.txcoord0.xy).xyz;
    res.xyz *= 0.2;

/// Mix bloom textures, for Local Operators
    res.w  = RenderTarget1024.Sample(Sampler1, IN.txcoord0.xy).w;
    res.w += RenderTarget512.Sample(Sampler1, IN.txcoord0.xy).w;
    res.w += RenderTarget256.Sample(Sampler1, IN.txcoord0.xy).w;
    res.w += RenderTarget128.Sample(Sampler1, IN.txcoord0.xy).w;
    res.w += RenderTarget64.Sample(Sampler1, IN.txcoord0.xy).w;
    res.w += RenderTarget32.Sample(Sampler1, IN.txcoord0.xy).w;
    res.w *= 0.2;

    res.xyz = msBloomDepth(res.xyz, IN.txcoord0.xy);    /// Requires 20_BloomDepth.fxh active

    res.xyz = msBloomControl(res.xyz, IN.txcoord0.xy);  /// Requires 21_BloomControl.fxh active

/// Clamp to avoid bugs
    res = max(res, 0.0);
    res = min(res, 16384.0);

  return res;
}


// TECHNIQUES
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Techniques are drawn one after another and they use the result of
// the previous technique as input color to the next one.  The number
// of techniques is limited to 255.  If UIName is specified, then it
// is a base technique which may have extra techniques with indexing
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//bloom example 1, draw in single pass, very performance heavy and low blurring range
technique11 SinglePassBloom <string UIName="Single pass bloom";>
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_Quad()));
    SetPixelShader(CompileShader(ps_5_0, PS_ExampleBloom1(TextureDownsampled, 1024.0, 1024.0)));
  }
}

//bloom example 2, draw using several techniques and several render targets
technique11 MultiPassBloom <string UIName="Multipass bloom"; string RenderTarget="RenderTarget1024";>
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_Quad()));
    SetPixelShader(CompileShader(ps_5_0, PS_Resize(TextureDownsampled, 1024.0, 1024.0)));
  }
}
technique11 MultiPassBloom1 <string RenderTarget="RenderTarget512";>
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_Quad()));
    SetPixelShader(CompileShader(ps_5_0, PS_Resize(RenderTarget1024, 1024.0, 512.0)));
  }
}
technique11 MultiPassBloom2 <string RenderTarget="RenderTarget256";>
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_Quad()));
    SetPixelShader(CompileShader(ps_5_0, PS_Resize(RenderTarget512, 512.0, 256.0)));
  }
}
technique11 MultiPassBloom3 <string RenderTarget="RenderTarget128";>
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_Quad()));
    SetPixelShader(CompileShader(ps_5_0, PS_Resize(RenderTarget256, 256.0, 128.0)));
  }
}
technique11 MultiPassBloom4 <string RenderTarget="RenderTarget64";>
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_Quad()));
    SetPixelShader(CompileShader(ps_5_0, PS_Resize(RenderTarget128, 128.0, 64.0)));
  }
}
technique11 MultiPassBloom5 <string RenderTarget="RenderTarget32";>
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_Quad()));
    SetPixelShader(CompileShader(ps_5_0, PS_Resize(RenderTarget64, 64.0, 32.0)));
  }
}
technique11 MultiPassBloom6  /// Last pass outputs to lens texture
{
  pass p0
  {
    SetVertexShader(CompileShader(vs_5_0, VS_Quad()));
    SetPixelShader(CompileShader(ps_5_0, PS_BloomMix()));
  }
}
